@@ -6,10 +6,8 @@ Engine
 Engine runs the simulation.
 """
 
-import sys
 import os
 import logging as log
-import warnings
 import pprint
 from typing import (
     Any, Dict, Optional, Union, Tuple, Callable)
@@ -340,7 +338,7 @@ class Engine:
             self._add_process_path(process, path)
 
     def emit_configuration(self) -> None:
-        """Emit configuration information to the emitter."""
+        """Emit experiment configuration."""
         data: Dict[str, Any] = {
             'time_created': self.time_created,
             'experiment_id': self.experiment_id,
@@ -348,22 +346,23 @@ class Engine:
             'description': self.description,
             'topology': self.topology,
             'processes': serialize_value(self.processes),
-            'state': serialize_value(self.state.get_config())
-        }
+            'state': serialize_value(self.state.get_config())}
         emit_config: Dict[str, Any] = {
             'table': 'configuration',
             'data': data}
+        self.emitter.emit(emit_config)
 
-        # get size of data for emit
-        data_bytes = sys.getsizeof(str(emit_config))
-        if data_bytes < 26000000:  # pymongo document size limit
-            self.emitter.emit(emit_config)
-        else:
-            warnings.warn('configuration size is too big for the emitter, '
-                          'discarding process parameters')
-            for process_id in emit_config['data']['processes'].keys():
-                emit_config['data']['processes'][process_id] = None
-            self.emitter.emit(emit_config)
+    def emit_data(self) -> None:
+        """Emit the current simulation state.
+        Only variables with ``_emit=True`` are emitted.
+        """
+        data = self.state.emit_data()
+        data.update({
+            'time': self.experiment_time})
+        emit_config = {
+            'table': 'history',
+            'data': serialize_value(data)}
+        self.emitter.emit(emit_config)
 
     def invoke_process(
             self,
@@ -556,18 +555,6 @@ class Engine:
                     path, deriver, 0)
                 self.apply_update(update.get(), store)
 
-    def emit_data(self) -> None:
-        """Emit the current simulation state.
-
-        Only variables with ``_emit=True`` are emitted.
-        """
-        data = self.state.emit_data()
-        data.update({
-            'time': self.experiment_time})
-        emit_config = {
-            'table': 'history',
-            'data': serialize_value(data)}
-        self.emitter.emit(emit_config)
 
     def send_updates(
             self,
