@@ -42,22 +42,19 @@ def test_rewire_ports() -> None:
     store = test_insert_process()
 
     # connect process1's port1 to the store at process3's port1
-    store['process1']['port1'] = store['process3']['port1']
-    assert store['process1']['port1'] == store['process3']['port1']
-
-    # this should give the same result
     store = test_insert_process()
-    store['process1', 'port1'] = store['process3', 'port1']
+    store['process1'].connect('port1', store['process3']['port1'])
     assert store['process1']['port1'] == store['process3']['port1']
 
     # connect process2's port2 to store store_A
     store = test_insert_process()
-    store['process2', 'port2'] = store['store_A']
+    store['process2'].connect('port2', store['store_A'])
     assert store['process2', 'port2', 'var_a'] == store['store_A', 'var_a']
 
     # turn variable 'var_a' into 'var_b'
     store = test_insert_process()
-    store['process2', 'port2', 'var_a'] = store['store_A', 'var_b']
+    store['process2'].connect(['port2', 'var_a'], store['store_A', 'var_b'])
+    # store['process2', 'port2', 'var_a'] = store['store_A', 'var_b']
     assert store['process2', 'port2', 'var_a'] == store['store_A', 'var_b']
 
 
@@ -74,8 +71,8 @@ def test_embedded_rewire_ports() -> None:
            store['down1', 'down2', 'store_C']
 
     # rewire process2 port2 to store_A, and assert change of wiring
-    store['down1', 'down2', 'process2', 'port2'] = \
-        store['down1', 'down2', 'store_A']
+    store['down1', 'down2', 'process2'].connect(
+        'port2', store['down1', 'down2', 'store_A'])
     assert store['down1', 'down2', 'process2', 'port2', 'var_a'] == \
            store['down1', 'down2', 'store_A', 'var_a']
 
@@ -83,7 +80,7 @@ def test_embedded_rewire_ports() -> None:
 def test_replace_process() -> None:
     """replace a process"""
     store = get_toy_store()
-    process4 = ToyProcess()  # {'name': 'process4'}
+    process4 = ToyProcess()
     store['process4'] = process4
     assert store['process4'].value == process4
 
@@ -91,7 +88,7 @@ def test_replace_process() -> None:
     store['process1'] = Qo({})
 
     # test if initial values are kept the same, and are not overwritten
-    store['A', 'a'] = 11
+    store.create(['A', 'a'], 11)
     store['process1'] = ToyProcess({})
     assert store['A', 'a'].value == 11
 
@@ -142,6 +139,14 @@ def test_set_value() -> None:
 def test_run_store_in_experiment() -> None:
     """put a store in an experiment and run it"""
     store = get_toy_store()
+
+    # retrieve the processes and topology
+    processes = store.get_processes()
+    topology = store.get_topology()
+    _ = processes  # set to _ to pass lint test
+    _ = topology
+
+    # run the experiment with a topology
     experiment = Engine({'store': store})
     experiment.update(10)
     data = experiment.emitter.get_data()
@@ -153,6 +158,59 @@ def test_run_store_in_experiment() -> None:
     print(data)
 
 
+def test_divide_store() -> None:
+    store = Store({})
+    store.create(['top', 'process1'], ToyProcess({}))
+    store.create(['top', 'store1', 'X'])
+    store['top', 'process1'].connect('port1', 'store1')
+
+    # divide store1 into two daughters
+    store['top'].divide({
+        'mother': 'store1',
+        'daughters': [
+            {'key': 'store2'},
+            {'key': 'store3'}
+        ]})
+
+    final_state = store.get_value()
+    assert 'store2' in final_state['top']
+    assert 'store3' in final_state['top']
+    assert 'store1' not in final_state['top']
+
+
+def test_update_schema() -> None:
+    store = Store({})
+    store.create(['top', 'process1'], ToyProcess({}))
+    store.create(['top', 'store1'], _updater='set')
+    assert store['top', 'store1'].updater == 'set', \
+        'updater is not set correctly'
+
+
+def test_port_connect() -> None:
+    # create the root
+    store = Store({})
+
+    # create a new store at a path
+    store.create(['top', 'store1'])
+    store.create(['top', 'store2'])
+
+    # create a process at a path
+    store.create(['top', 'process1'], ToyProcess({}))
+    store.create(['top', 'process2'], ToyProcess({}))
+
+    # connect port using a relative path
+    store['top', 'process1'].connect('port1', 'store1')
+
+    # connect using store target through a different port
+    store['top', 'process1'].connect('port2', store['top', 'process1', 'port1'])
+
+    # connect using absolute path
+    store['top', 'process1'].connect('port2', ('top', 'store2'), absolute=True)
+
+    assert store['top', 'process1'].topology == {
+        'port1': ('store1',), 'port2': ('store2',)}
+
+
 test_library = {
     '1': test_insert_process,
     '2': test_rewire_ports,
@@ -162,6 +220,9 @@ test_library = {
     # '6': test_connect_to_new_store,
     '7': test_set_value,
     '8': test_run_store_in_experiment,
+    '9': test_divide_store,
+    '10': test_update_schema,
+    '11': test_port_connect,
 }
 
 
